// randomtones George Thomas, W5JDX  11/24
// updated with comments 11/16/24
#include "analogWave.h"// include the AnalogWave library:
analogWave wave(DAC);

// links to help with LED frame displays
// https://docs.arduino.cc/tutorials/uno-r4-wifi/led-matrix/
// https://ledmatrix-editor.arduino.cc/
#include "animation.h"          //animation created with ledmatrix-editor
#include "ArduinoGraphics.h"
#include "Arduino_LED_Matrix.h"
#include <OPAMP.h>
ArduinoLEDMatrix matrix;        //create LED Matrix class

const int speakerPin = A0;      //the pin number for audio output
int progMode = 0;               //program routine to run
int loopCount = 0;              //count trips through loop for test guess mode

void setup() {
  wave.sine(10);                //create 10 Hz sine wave output
  wave.stop();                  //stop the ouput
  OPAMP.begin(OPAMP_SPEED_LOWSPEED);  //start OPAMP mode at lower gain
  pinMode(1,INPUT_PULLUP);      //set PinModes for input with internal pull up resistors
  pinMode(2,INPUT_PULLUP);
  pinMode(3,INPUT_PULLUP);

  analogReadResolution(14);     //change to 14-bit resolution since we can
  
  matrix.begin();               //start LED Matrix
  matrix.beginDraw();           //start drawing
  matrix.stroke(0xFFFFFFFF);    //color of stroke. We can't really change it
  matrix.textScrollSpeed(50);   //speed to scroll text
  const char text[] = "    AMATEURLOGIC.TV";  //startup text to display
  matrix.textFont(Font_5x7);    //font size
  matrix.beginText(0, 1, 0xFFFFFF); //start printing text
  matrix.println(text);         //print text from above
  matrix.endText(SCROLL_LEFT);  //scroll to the left and end
  matrix.endDraw();             //stop drawing
}

void loop() {
  char text1[] = "1kH";         //text to display for 2 of the 3 modes
  char text2[] = "ABCD";        //character array of possible answers for testGuess function
  int intGuess;
  int btn1Val = digitalRead(1); //read digital inputs to see which if any button was pressed.
  int btn2Val = digitalRead(2);
  int btn3Val = digitalRead(3);
  if(btn1Val == LOW)            //set progMode for button pressed
      progMode = 1;             //play 1 kHz sine wave
  else if(btn2Val == LOW)
    progMode = 2;               //play random tones and LED frames
  else if(btn3Val == LOW)
    progMode = 3;               //play random tones and LED frames, then stop and randomly display A,B,C, or D

  switch(progMode){   //1 kHz Tone
    case 1:
      matrix.textFont(Font_4x6);
      matrix.beginText(0, 1, 0xFFFFFF);
      matrix.println(text1);
      matrix.endText();
      oneKtone();
      progMode=0;     //only need to call once
      break;
    case 2:           //random tones, continue to call each loop
      randTones();
      break;
    case 3:           //test guess function
      if(loopCount < 20){       //run through loop 20 times, playing random tones
        loopCount = loopCount + 1;
        randTones();
        break;
      }
      wave.stop();              //stop playing tones and display random answer
      intGuess = random(0,4);   //choose a random number for A,B,C, or D
      matrix.textFont(Font_5x7);
      matrix.beginText(4, 1, 0xFFFFFF);
      matrix.println(text2[intGuess]);
      matrix.endText();
      loopCount = 0;            //reset loop counter
      progMode = 0;             //end this function
      break;
    default:
      break;
 }
}

void oneKtone(){
  wave.freq(1000);              //play 1 kHz tone
}

void randTones(){               //play random tones. Also used by testGuess function 
  int speedPot =analogRead(A5); //check voltage from pot to determine speed of function repeats
  speedPot = speedPot/164;
  int randani=random(0,15);     //choose one of the 15 LED frames in animation.h
  matrix.loadFrame(animation[0,randani]);
  int freq;
  int randInt = random(200,1500);//choose random tone from 200 to 1500 Hz
  wave.freq(randInt);           //play the tone
  delay(speedPot);              //wait
  matrix.clear();               //clear LED display
  matrix.endDraw();             //stop drawing
}
